library ac_dart.host_model;

import 'package:path/path.dart' as path;
import 'package:option/option.dart';

class AddonHost {
  final String key;
  final Uri baseUrl;
  final Option<String> addonSecret;
  
  AddonHost(this.key, this.baseUrl, [String addonSecret])
      : this.addonSecret = new Option(addonSecret);
}



class ProductType {
  final int ordinal;
  final String literal;

  const ProductType(this.ordinal, this.literal);

  static const ProductType jira = const ProductType(1, "jira");
  static const ProductType confluence = const ProductType(2, "confluence");

  static ProductType fromLiteral(String literal) {
    // TODO: ugly impl
    switch (literal) {
      case "jira":
        return jira;
      case "confluence":
        return confluence;
      default:
        return null;
    }
  }

  String toString() => literal;
}

class AcHostDetails extends Object with _AcHostDetailsMixin {
  final String key;
  final ProductType productType;
  final String publicKey;
  final Uri baseUrl;

  AcHostDetails(this.key, this.productType, this.publicKey, this.baseUrl);

  String toString() => 'AcHostDetails[$key, $productType, $baseUrl]';

  AcHostDetails.fromJson(Map<String, Object> jsonMap): this(jsonMap['key'],
      ProductType.fromLiteral(jsonMap['productType']), jsonMap['publicKey'],
      Uri.parse(jsonMap['baseUrl']));

}

class AcHostRegistration extends Object with _AcHostRegistrationMixin {
  final int id;
  final AcHostDetails hostDetails;
  final String sharedSecret;

  AcHostRegistration(this.id, this.hostDetails, this.sharedSecret);

  AcHostRegistration.fromJson(Map<String, Object> jsonMap)
      : this(0, new AcHostDetails.fromJson(jsonMap['hostDetails']),
          jsonMap['sharedSecret']);

}

class MutableAcHostDetails extends Object with _AcHostDetailsMixin implements
    AcHostDetails {
  String key;
  ProductType productType;
  String publicKey;
  Uri baseUrl;

  AcHostDetails toImmutable() => new AcHostDetails(key, productType, publicKey,
      baseUrl);
}

class MutableAcHostRegistration extends Object with _AcHostRegistrationMixin
    implements AcHostRegistration {
  int id;
  MutableAcHostDetails hostDetails;
  String sharedSecret;

  AcHostRegistration toImmutable() => new AcHostRegistration(id,
      hostDetails.toImmutable(), sharedSecret);
}

abstract class _AcHostDetailsMixin {
  // TODO: remove from here
  static const String _consumerInfoPath =
      "/plugins/servlet/oauth/consumer-info";
  String get key;
  ProductType get productType;
  String get publicKey;
  Uri get baseUrl;

  Uri get consumerInfoUrl => Uri.parse(path.join(baseUrl.origin,
      _consumerInfoPath));

  Map<String, Object> toJson() => {
    'key': key,
    'productType': productType.literal,
    'publicKey': publicKey,
    'baseUrl': baseUrl.toString()
  };

}

abstract class _AcHostRegistrationMixin {
  int get id;
  MutableAcHostDetails get hostDetails;
  String get sharedSecret;

  Map<String, Object> toJson() => {
    'sharedSecret': sharedSecret, // TODO: encrypt??
    'hostDetails': hostDetails.toJson()
  };

  String toString() => 'AcHostRegistration[$id, $hostDetails]';

}
