library ac_dart.host_registration_persistence;

import 'package:atlassian_connect_host/host_model.dart';
import 'dart:async';
import 'package:logging/logging.dart';
export 'package:atlassian_connect_host/host_model.dart' show AcHostRegistration;
export 'dart:async' show Future;
import 'package:option/option.dart';
export 'package:option/option.dart';

final Logger _log = new Logger('AcHostRegistrationRepository');

typedef AcHostRegistrationRepository AcHostRegistrationRepositoryFactory();

/**
 * A Repository (DAO) for [AcHostRegistration] data
 */
abstract class AcHostRegistrationRepository {

  Future<AcHostRegistration> create(AcHostRegistration hostRegistration);

  Future<Option<AcHostRegistration>> findRegistrationByHostKey(String hostKey);
}

class AcHostRegistrationInMemoryTestRepository implements
    AcHostRegistrationRepository {

  final Map<String, AcHostRegistration> _repo = {};

  @override
  Future<AcHostRegistration> create(AcHostRegistration hostRegistration) {
    _repo[hostRegistration.hostDetails.key] = hostRegistration;
    _log.fine('persisted $hostRegistration');
    return new Future.value(hostRegistration);
  }

  @override
  Future<Option<AcHostRegistration>> findRegistrationByHostKey(String hostKey) =>
      new Future.value(new Option(_repo[hostKey]));
}

AcHostRegistrationRepositoryFactory createInMemoryAcHostRegistrationRepository() {
  return () => new AcHostRegistrationInMemoryTestRepository();
}
