library ac_dart.host_event_model.test;

import 'package:atlassian_connect_host/host_model.dart';
import 'package:atlassian_connect_host/host_event_model.dart';
import 'package:unittest/unittest.dart';
import 'dart:convert';

main() {

  group('[Json]', () {
    final clientKey = '1234567890';
    final publicKey = 'MIGf....ZRWzwIDAQAB';
    final sharedSecret = '1ad6f705-fe0b-4111-9551-7ce5d81d2884';
    final baseUrl = 'http://storm.dyn.syd.atlassian.com:2990/jira';
    final productType = 'jira';
    final eventType = 'install';
    final jsonString =
        ''' 
          {
              "key": "atlassian-connect-jira-addon-jwt",
              "clientKey": "$clientKey",
              "publicKey": "$publicKey",
              "sharedSecret": "$sharedSecret",
              "baseUrl": "$baseUrl",
              "productType": "jira",
              "eventType": "installed"
          }
        ''';
    final json = JSON.decode(jsonString);

    group('[unmarshalling]', () {
      final installedEvent = new InstallationEvent.fromJson(json);
      final registration = installedEvent.hostRegistration;
      hostDetails() => registration.hostDetails;
      
      test('eventType parses correctly', () {
        expect(installedEvent.eventType, equals(AcHostEventType.installed));
      });

      test('hostRegistration not null', () {
        expect(registration, isNotNull);
      });
      
      test('sharedSecret parses correctly', () {
        expect(registration.sharedSecret, equals(sharedSecret));
      });
      
      test('hostDetails not null', () {
        expect(hostDetails(), isNotNull);
      });
      
      test('clientKey parses correctly', () {
        expect(hostDetails().key, equals(clientKey));
      });
      
      test('publicKey parses correctly', () {
        expect(hostDetails().publicKey, equals(publicKey));
      });
      
      test('baseUrl parses correctly', () {
        expect(hostDetails().baseUrl, equals(Uri.parse(baseUrl)));
      });
      
      test('productType parses correctly', () {
        expect(hostDetails().productType, equals(ProductType.jira));
      });
    });
  });
}
